//+------------------------------------------------------------------+
//|                                              Exp_EF_distance.mq5 |
//|                             Copyright  2012,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+------------------------------------------------------------------+
#property copyright "Copyright  2012, Nikolay Kositsin"
#property link      "farria@mail.redcom.ru"
#property version   "1.00"
//+----------------------------------------------+
//|    CXMA                        |
//+----------------------------------------------+
#include <SmoothAlgorithms.mqh> 
//+----------------------------------------------+
//|                        |
//+----------------------------------------------+
enum Applied_price_ // 
  {
   PRICE_CLOSE_ = 1,     //Close
   PRICE_OPEN_,          //Open
   PRICE_HIGH_,          //High
   PRICE_LOW_,           //Low
   PRICE_MEDIAN_,        //Median Price (HL/2)
   PRICE_TYPICAL_,       //Typical Price (HLC/3)
   PRICE_WEIGHTED_,      //Weighted Close (HLCC/4)
   PRICE_SIMPL_,         //Simpl Price (OC/2)
   PRICE_QUARTER_,       //Quarted Price (HLOC/4) 
   PRICE_TRENDFOLLOW0_,  //TrendFollow_1 Price 
   PRICE_TRENDFOLLOW1_,  //TrendFollow_2 Price
   PRICE_DEMARK_         //Demark Price 
  };
//+----------------------------------------------+
//|                        |
//+----------------------------------------------+
enum Volatility // 
  {
   V1,     //1
   V2,     //2
   V3      //3
  };
//+----------------------------------------------+
//|            |
//+----------------------------------------------+
input double MM=-0.1;             //      ,   -  
input int    StopLoss_=1000;      //  
input int    TakeProfit_=2000;    //  
input int    Deviation_=10;       //.    
input bool   BuyPosOpen=true;     //    
input bool   SellPosOpen=true;    //    
input bool   BuyPosClose=true;     //    
input bool   SellPosClose=true;    //    
//----
input ENUM_TIMEFRAMES InpInd_Timeframe=PERIOD_H4; // 
//----
input uint SignalBar=1;//     
//+----------------------------------------------+
//|    EF_distance     |
//+----------------------------------------------+
input int XLength=10;                       //                     
input uint Power=2.0;                       // 
input Applied_price_ IPC=PRICE_CLOSE;       // 
//+----------------------------------------------+
//|    Flat-Trend      |
//+----------------------------------------------+
input uint StDevPeriod=20;                  // StDev 
input Smooth_Method StDev_Method=MODE_LWMA; //  StDev
input uint StDevLength=5;                   //  StDev                    
input int StDevPhase=15;                    //  StDev
input uint ATRPeriod=20;                    // StDev   
input Smooth_Method ATR_Method=MODE_LWMA;   //  ATR
input uint ATRLength=5;                     //  ATR
input int ATRPhase=15;                      //  ATR
input Volatility Volatil=V3;                //    
//+----------------------------------------------+
//----          
int TimeShiftSec;
//----      
int InpInd_Handle1,InpInd_Handle2;
//----      
int min_rates_total;
//+------------------------------------------------------------------+
//                                                  | 
//+------------------------------------------------------------------+
#include <TradeAlgorithms.mqh>
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//----    EF_distance
   InpInd_Handle1=iCustom(Symbol(),InpInd_Timeframe,"EF_distance",XLength,Power,IPC,0,0);
   if(InpInd_Handle1==INVALID_HANDLE) Print("      EF_distance");

//----    Flat-Trend
   InpInd_Handle2=iCustom(Symbol(),InpInd_Timeframe,"Flat-Trend",
                          StDevPeriod,StDev_Method,StDevLength,StDevPhase,ATRPeriod,ATR_Method,ATRLength,ATRPhase,0);
   if(InpInd_Handle2==INVALID_HANDLE) Print("      Flat-Trend");

//----          
   TimeShiftSec=PeriodSeconds(InpInd_Timeframe);

//----     
   min_rates_total=int(2*XLength);
   min_rates_total+=int(3+SignalBar);

   CXMA XMA;
   int min_rate_1=int(StDevPeriod)+XMA.GetStartBars(StDev_Method,StDevLength,StDevPhase);
   int min_rate_2=int(ATRPeriod)+XMA.GetStartBars(ATR_Method,ATRLength,ATRPhase);
   int min_rates_=MathMax(min_rate_1,min_rate_2)+1;

   min_rates_total=MathMax(min_rates_,min_rates_total);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//----
   GlobalVariableDel_(Symbol());
//----
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//----       
   if(BarsCalculated(InpInd_Handle1)<min_rates_total || BarsCalculated(InpInd_Handle2)<min_rates_total) return;

//----       IsNewBar()  SeriesInfoInteger()  
   LoadHistory(TimeCurrent()-PeriodSeconds(InpInd_Timeframe)-1,Symbol(),InpInd_Timeframe);

//----   
   double Value[3],Vol[1];
//----   
   static bool Recount=true;
   static bool BUY_Open=false,BUY_Close=false;
   static bool SELL_Open=false,SELL_Close=false;
   static datetime UpSignalTime,DnSignalTime;
   static CIsNewBar NB;

//+----------------------------------------------+
//|                  |
//+----------------------------------------------+
   if(!SignalBar || NB.IsNewBar(Symbol(),InpInd_Timeframe) || Recount) //     
     {
      //----   
      BUY_Open=false;
      SELL_Open=false;
      BUY_Close=false;
      SELL_Close=false;
      Recount=false;

      //----      
      if(CopyBuffer(InpInd_Handle1,0,SignalBar,3,Value)<=0) {Recount=true; return;}
      if(CopyBuffer(InpInd_Handle2,0,SignalBar,1,Vol)<=0) {Recount=true; return;}

      //----    
      if(Value[1]<Value[2])
        {
         if(BuyPosOpen && Value[0]>Value[1] && Vol[0]>=Volatil) BUY_Open=true;
         if(SellPosClose) SELL_Close=true;
         UpSignalTime=datetime(SeriesInfoInteger(Symbol(),InpInd_Timeframe,SERIES_LASTBAR_DATE))+TimeShiftSec;
        }

      //----    
      if(Value[1]>Value[2])
        {
         if(SellPosOpen && Value[0]<Value[1] && Vol[0]>=Volatil) SELL_Open=true;
         if(BuyPosClose) BUY_Close=true;
         DnSignalTime=datetime(SeriesInfoInteger(Symbol(),InpInd_Timeframe,SERIES_LASTBAR_DATE))+TimeShiftSec;
        }
     }

//+----------------------------------------------+
//|                              |
//+----------------------------------------------+
//----  
   BuyPositionClose(BUY_Close,Symbol(),Deviation_);

//----     
   SellPositionClose(SELL_Close,Symbol(),Deviation_);

//----  
   BuyPositionOpen(BUY_Open,Symbol(),UpSignalTime,MM,0,Deviation_,StopLoss_,TakeProfit_);

//----  
   SellPositionOpen(SELL_Open,Symbol(),DnSignalTime,MM,0,Deviation_,StopLoss_,TakeProfit_);
//----
  }
//+------------------------------------------------------------------+
